////////////////////////////////////////////////////////////////////////////////
// File         : GraphPrint.cpp
// Subsystem    : Graph
// Component    : graphs
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : implementation of the class GraphPrint
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 30.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
// 30.05.06    Josef Huebl       some extensions
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//     
//     
////////////////////////////////////////////////////////////////////////////////

#include "GraphPrint.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

GraphPrint::GraphPrint( const char *name ) : ObjPrint( "Graph", name )
{
}


GraphPrint::GraphPrint( const GraphPrint &g, const char *name ) : ObjPrint( "Graph", name )
{
	Copy( g );
};


GraphPrint::GraphPrint( const Graph &g, const char *name ) : ObjPrint( "Graph", name )
{
	Graph::AddCopyOf( g );
	Renumber( FirstNode() );
};


GraphPrint::~GraphPrint()
{

}


//////////////////////////////////////////////////////////////////////
// public
//////////////////////////////////////////////////////////////////////


GraphPrint& GraphPrint::operator=( const GraphPrint &g)
{
	if (this != &g)
		Copy( g );

	return *this;
}


void GraphPrint::Copy( const GraphPrint &g)
{
	ObjPrint::Copy( g ); 
	Graph::DeleteAllNodes();
	Graph::AddCopyOf( g );
	NodesNumbers::Copy( g, FirstNode(), g.FirstNode() );
}


Node GraphPrint::CreateNode(void)
{
	Node n = Graph::CreateNode();
	NewNodeID( n );
	return n;
}



//////////////////////////////////////////////////////////////////////
// private
//////////////////////////////////////////////////////////////////////

void GraphPrint::PrintNeighbours( Node &n ) const
{
	Edge e;

	printf( "  %2u", GetNodeID( n ) );

	e = n.FirstOutgoingEdge();

	if ( e )
		printf( " ->" );

	while ( e )
	{
		printf( " %2u", GetNodeID( e.TargetNode() ) );
		e = e.NextOutgoingEdge();
	}
}


void GraphPrint::PrintBody( void ) const
{
	Edge e;
	Node n = FirstNode();

	while ( n )
	{
		PrintNeighbours( n );
		printf( "\n" );
		n = n.NextNode();
	}
}



// ****************************** RelationPrint ************************************

void RelationPrint::PrintBody( void ) const
{
	NodePair current = First();
	int newline = 1;

	printf( "  " );

	while ( current )
	{
		printf ("(%02d %02d) ", m_nodesNumbers->GetNodeID( current.First() ), m_nodesNumbers->GetNodeID( current.Second() ) );

		if ( newline == 10)
		{
			printf( "\n  " );
			newline = 0;
		}

		newline++;
		current = Next();
	}

	printf( "\n\n" );
}

// ****************************** NodeVectorPrint ************************************

void NodeVectorPrint::PrintBody( void ) const
{
	Node current = First();
	int newline = 1;

	printf( "  " );

	while ( current )
	{
		printf ("%3d ", m_nodesNumbers->GetNodeID( current ) );

		if ( newline == 10)
		{
			printf( "\n  " );
			newline = 0;
		}

		newline++;
		current = Next();
	}

	printf( "\n\n" );
}




// ****************************** EdgeVectorPrint ************************************

void EdgeVectorPrint::PrintBody( void ) const
{
	Edge current = First();
	int newline = 1;

	while ( current )
	{
		printf ("(%02d, %02d) ", m_nodesNumbers->GetNodeID( current.SourceNode() ), m_nodesNumbers->GetNodeID( current.TargetNode() ) );

		if ( newline == 10)
		{
			printf( "\n" );
			newline = 0;
		}

		newline++;
		current = Next();
	}

	printf( "\n\n" );
}


