////////////////////////////////////////////////////////////////////////////////
// File         : HnetPrint.h
// Subsystem    : Graph
// Component    : H-net
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : definition of the class HnetPrint
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 19.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//     
// This class adds unique numbers to the nodes the graph of a h-net and
// allows to print out a h-net with a single call of Print().
//     
////////////////////////////////////////////////////////////////////////////////


#if !defined(AFX_HNETPRINT_H__B54EC041_B301_4F23_920B_83E93C0D9895__INCLUDED_)
#define AFX_HNETPRINT_H__B54EC041_B301_4F23_920B_83E93C0D9895__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "ObjPrint.h"
#include "NodesNumbers.h"
#include "Hnet.h"

class HnetUniversePrint;

class HnetPrint : public ObjPrint, public NodesNumbers, public Hnet  
{
public:
	HnetPrint( HnetUniversePrint *universe, const char *name = ObjPrintNoname );
	HnetPrint( HnetPrint &hnet, const char *name = ObjPrintNoname);
	HnetPrint( HnetUniversePrint *universe, Hnet &hnet, const char *name = ObjPrintNoname );
	virtual ~HnetPrint();
	HnetPrint& operator=( HnetPrint &hnet);

	// edit graph functions
	Node AddOuterEntry( void ) { Node n = Hnet::AddOuterEntry(); RenumberYX( Root() ); return n; };
	Node AddOuterExit( void ) { Node n = Hnet::AddOuterExit();RenumberYX( Root() ); return n; };
	void DelOuterEntry(  Node &y ) { Hnet::DelOuterEntry( y ); RenumberYX( Root() ); };
	void DelOuterExit(  Node &x ) { Hnet::DelOuterExit( x ); RenumberYX( Root() ); };

	Node AddSubcentre( const IsoTypeNode &isoType ) { Node s = Hnet::AddSubcentre( isoType ); RenumberSubcentre( s ); return s; };
	Node CopySubcentre( const Node &s ) { return AddSubcentre( GetIsoType( s ) ); };
	void ChangeIsoType( const Node &s, const IsoTypeNode &isoType ) { Hnet::ChangeIsoType(s, isoType ); RenumberSubcentre( s ); };
	Node Expand( Node &s ) { Node ss = Hnet::Expand( s ); RenumberNewSubcentre(); return ss; };
	Node ExpandAll( void ) { Node s = Hnet::ExpandAll(); RenumberNewSubcentre(); return s; };
	void MakeFlat( void ) { Hnet::MakeFlat(); RenumberNewSubcentre(); };

	void PrintBody( void ) const;
	char *Str( const Node &n) const;

private:
	const HnetUniversePrint *m_universePrint; 

private:
	void Copy( HnetPrint &hnet);

	void RenumberNewSubcentre( void );
	void RenumberYX( const Node &s );
	void RenumberSubcentre( const Node &s );

	void PrintSubcenter( const Node &s ) const;
	void PrintConnections( const Node &s ) const;

};

#endif // !defined(AFX_HNETPRINT_H__B54EC041_B301_4F23_920B_83E93C0D9895__INCLUDED_)
